*-----------------------------------------------------------
* Program    :
* Written by :
* Date       :
* Description:
*-----------------------------------------------------------
ROM	    EQU $0
ROMTOP      EQU $77FFF
SERIN       EQU $78000
SEROUT      EQU $7A000
SERSTATUS   EQU $7C000
DOUT        EQU $7E000
RAM         EQU $80000
RAMTOP      EQU $FFFFF

; status register values
INT_ON   EQU  $2000    ; BOTH, SET SUPERVISOR MODE, S=1
INT_OFF  EQU  $2700
SUPERVISOR_BIT EQU 5
TRACE_BIT EQU 7

; serial communication
RDF    EQU 0
TXE    EQU 1

CR      EQU 13
LF      EQU 10
SP      EQU 32
BS      EQU 8
RS      EQU $1E
ESC     EQU $1B
BIT_ESC      EQU 0              ; ESC BIT POSITION #0

	ORG	RAM 
START:				
	;MOVE.W	#$2700,SR	; interrupts off	
	MOVE.L  #RAMTOP+1,SP
	
	LEA.L WELCOME.L,A4 
	BSR PSTR
	
	LEA.L MANU_ID.L,A4 
	BSR PSTR
	JSR FLASH_GET_MANUFACTURER_ID
	JSR OUT2X
	JSR NEW_LINE
	
	MOVE.B #$01,D1
	CMP D1,D0
	BEQ N1
	LEA.L ID_ERROR.L,A4
	JSR PSTR
	MOVE.B D1,D0
	JSR OUT2X
	JSR NEW_LINE 
	JMP DIE
	
N1:	LEA.L DEV_ID.L,A4 
	BSR PSTR
	JSR FLASH_GET_DEVICE_ID
	JSR OUT2X
	JSR NEW_LINE

	MOVE.B #$A4,D1
	CMP D1,D0
	BEQ N2
	LEA.L ID_ERROR.L,A4
	JSR PSTR
	MOVE.B D1,D0
	JSR OUT2X
	JSR NEW_LINE 
	JMP DIE

N2:
	JSR READ_BINARY
	JSR FLASH_ERASE_CHIP
	; A0 = source buffer start address (in RAM)
	; A1 = source buffer end address + 1
	; A2 = dest buffer start address (in Flash ROM)
	MOVE.L A6,A0
	MOVE.L A5,A1
	MOVE.L #0,A2	
	JSR FLASH_PROGRAM
	JMP DIE

FLASH_GET_MANUFACTURER_ID:
	MOVE.B #$AA,ROM+$555
	MOVE.B #$55,ROM+$2AA
   	MOVE.B #$90,ROM+$555
   	MOVE.B ROM+$0, D0
   	JSR FLASH_RESET
   	RTS
   
FLASH_GET_DEVICE_ID:
	MOVE.B #$AA,ROM+$555
	MOVE.B #$55,ROM+$2AA
   	MOVE.B #$90,ROM+$555
   	MOVE.B ROM+$001,D0
   	JSR FLASH_RESET
   	RTS
	
FLASH_RESET:
	MOVE.B #$F0,ROM
	RTS

; D0 = sector number (0-7)
FLASH_ERASE_SECTOR:
	CMP.B #8,D0
	BGE FLASH_SECTOR_RANGE
	LEA.L ERASING_SECTOR.L,A4
	JSR PSTR
	JSR OUT1X
	LEA.L DOT_DOT_DOT.L,A4
	JSR PSTR
	MOVE.B #$AA,ROM+$555
	MOVE.B #$55,ROM+$2AA
   	MOVE.B #$80,ROM+$555
	MOVE.B #$AA,ROM+$555
	MOVE.B #$55,ROM+$2AA
	; sector address goes in A18-A16, sectors are 64K
	MOVE.L #0,D1
	ADD.B D0,D1
	SWAP D1
	ADD.L #ROM,D1
	MOVE.L D1,A4
	MOVE.B #$30,(A4)
	
	; wait until it's done. DQ7 will go to 1.
FLASH_ERASE_WAIT:	
	BTST.B #7,(A4)
	BEQ FLASH_ERASE_WAIT
	
	LEA.L DONE.L,A4
	JSR PSTR
	RTS
	
FLASH_SECTOR_RANGE:
	LEA.L SECTOR_RANGE.L,A4
	JSR PSTR
	JMP DIE
	
FLASH_ERASE_CHIP:
	LEA.L ERASING_CHIP.L,A4
	JSR PSTR
	MOVE.B #$AA,ROM+$555
	MOVE.B #$55,ROM+$2AA
   	MOVE.B #$80,ROM+$555
	MOVE.B #$AA,ROM+$555
	MOVE.B #$55,ROM+$2AA
	MOVE.B #$10,ROM+$555
	MOVE.L #ROM,A4
	BRA FLASH_ERASE_WAIT

; A0 = source buffer start address (in RAM)
; A1 = source buffer end address + 1
; A2 = dest buffer start address (in Flash ROM)
FLASH_PROGRAM:
	LEA.L PROGRAMMING_CHIP.L,A4
	JSR PSTR
	MOVE.B #'.',D3
FLASH_PROGRAM_NEXT_BYTE:
	MOVE.B #$AA,ROM+$555
	MOVE.B #$55,ROM+$2AA
   	MOVE.B #$A0,ROM+$555
   	MOVE.B (A0)+,(A2)+
FLASH_PROGRAM_WAIT
	; while programming is in progress, successive reads to the same address will see DQ6 toggle
	MOVE.B ROM,D0
	MOVE.B ROM,D1
	CMP.B D0,D1
	BNE FLASH_PROGRAM_WAIT
	; finished?
	MOVE.L A0,D2
	MOVE.B D2,D4
	BNE NO_ACK
	MOVE.B D3,SEROUT
NO_ACK
   	CMP.L  A0,A1
   	BNE FLASH_PROGRAM_NEXT_BYTE
   	LEA.L DONE.L,A4
	JSR PSTR
	RTS

READ_BINARY LEA.L LOADB.L,A4
         BSR PSTR
         LEA.L STARTA.L,A4
         BSR PSTR
         BSR GET_ADDRESS
         MOVEA.L D6,A5
         MOVEA.L D6,A6
         BSR NEW_LINE
         LEA.L LOADB1.L,A4
         BSR PSTR
         CLR.L D2 ; count of bytes

WAITBIN
         BSR CINREADY
         BNE WAITBIN
         
         BSR CINS
         CMP.B #ESC,D0
         BNE GOTBIN
         RTS

GETBIN   ;BSR CINS
         MOVE.B SERIN,D0
GOTBIN   MOVE.B D0,(A5)+
         ;MOVE.B D0,DOUT.L  ; INDICATOR WHILE LOADING
         ADDQ.L #1, D2
         MOVE.B D2, D3
         BNE NOACK
         ;MOVE.B #'_',D0
         ;BSR COUT
         MOVE.B #'_',SEROUT
NOACK    ; quick test for next byte ready
         BTST.B #RDF,SERSTATUS
         BEQ GETBIN
         ; busy loop for next byte
         MOVE.L #20000, D1 ; reset timeout
	
WAITBIN2 
         ;BSR CINREADY
         BTST.B #RDF,SERSTATUS
         BEQ GETBIN
         DBF D1,WAITBIN2
         
         ; done
         BSR NEW_LINE
         MOVE.L D2,D0
         BSR OUT6X
         LEA.L NUMBER.L,A4
         BSR PSTR
         BSR NEW_LINE
         RTS

; PRINT HEX 
; OUT1X = PRINT ONE HEX
; OUT2X = PRINT TWO
; OUT4X = PRINT FOUR
; OUT8X = PRINT EIGHT
; ENTRY: D0

OUT1X        MOVE.B D0,-(SP)    ;SAVE D0
             AND.B #$F,D0
             ADD.B #'0',D0
             CMP.B #'9',D0
             BLS.S   OUT1X1
             ADD.B #7,D0
OUT1X1       BSR COUT
             MOVE.B (SP)+,D0    ;RESTORE D0
             RTS

OUT2X        ROR.B #4,D0
             BSR.S OUT1X
             ROL.B #4,D0
             BRA OUT1X

OUT4X        ROR.W #8,D0
             BSR.S OUT2X
             ROL.W #8,D0
             BRA.S OUT2X

OUT6X        SWAP.W D0        ; OUT 24-BIT HEX NUMBER
             BSR.S OUT2X
             SWAP.W D0
             BRA.S OUT4X    
	
DIE
          MOVE.L $4,A0
          JMP (A0)
        

COUT      MOVE.B #$FD,DOUT	  
	  BTST.B #TXE,SERSTATUS
          BNE.S  COUT
          MOVE.B D0,SEROUT
          MOVE.B #$FF,DOUT
          RTS


CINS      MOVE.B #$FE,DOUT
	  BTST.B #RDF,SERSTATUS
          BNE.S  CINS
          MOVE.B SERIN,D0
          MOVE.B #$FF,DOUT
          RTS


CIN      MOVE.B #$FE,DOUT
	 BTST.B #RDF,SERSTATUS
         BNE.S  CIN
         MOVE.B SERIN,D0
         BSR COUT
         MOVE.B #$FF,DOUT
         RTS
 
CINREADY BTST.B #RDF,SERSTATUS
         RTS
         
NEW_LINE MOVE.L D0,-(SP)
         MOVE.B #CR,D0
         BSR COUT
         MOVE.B #LF,D0
         BSR COUT
         MOVE.L (SP)+,D0
         RTS
         

; GET_ADDRESS
; EXIT: D6 CONTAINS 24-BIT ADDRESS

GET_ADDRESS   CLR.L D6
              BSR GET_HEX

GET_ADDRESS1  MOVE.B D1,D6
              ROL.L #8,D6
              BSR GET_HEX
              MOVE.B D1,D6
              ROL.L #8,D6
              BSR GET_HEX
              MOVE.B D1,D6
              RTS
             
GET_HEX  BSR CIN

         ;CMP.B #' ',D0         ; IF BIT_ESC PRESSED
         ;BNE.S GET_HEX1
         ;BSET.B #BIT_ESC,FLAG(A6)
         ;RTS


GET_HEX1 CMP.B #CR,D0
         BNE.S GET_HEX2
         ;BSET.B #1,FLAG(A6)       ; ENTER PRESSED
         RTS


GET_HEX2 BSR TO_HEX
         ROL.B #4,D1
         MOVE.B D1,D2
         BSR CIN
         BSR TO_HEX
         ADD.B D2,D1
         RTS
 
 ; CONVERT ASCII LETTER TO 8-BIT VALUE

TO_HEX SUBI.B #$30,D0
       CMPI.B #$A,D0
       BMI  ZERO_TO_NINE
       AND.B #$DF,D0
       SUBI.B #7,D0

ZERO_TO_NINE

       MOVE.B D0,D1

        RTS
        
; A4 POINTED TO FIRST BYTE
; END WITH 0

PSTR     MOVE.B (A4)+,D3
         CMP.B  #0,D3
         BEQ.S PSTR1
         MOVE.B D3,SEROUT
         BRA.S PSTR

PSTR1    RTS
 
* Variables and Strings

WELCOME dc.b 'Checking Flash ID...',13, 10, 0
MANU_ID dc.b 'Manufacturer ID: ', 0
DEV_ID  dc.b 'Device ID: ', 0
ID_ERROR dc.b 'ABORT - ID does not match expected value ', 0
SECTOR_OK dc.b 'Sector number OK', 13, 10, 0
SECTOR_RANGE dc.b 'Sector number is outside valid range', 13, 10, 0
ERASING_SECTOR dc.b 'Erasing sector ', 0
DOT_DOT_DOT dc.b '...', 0
DONE    dc.b 'done.', 13, 10, 0
ERASING_CHIP dc.b 'Erasing Flash chip...', 0
PROGRAMMING_CHIP dc.b 'Programming Flash chip...', 0
LOADB  dc.b 'Reprogram Flash ROM data',13,10,0
LOADB1 dc.b 'Ready to receive. Terminates after inactivity. (cancel: ESC)', 13,10,0
STARTA dc.b 13,10,10,'start address=',0
NUMBER dc.b ' (hex) bytes read', 13, 10, 0

	END	START		; last line of source



